@@ -5,13 +5,16 @@ from .comp_building import *
 from .comp_metamodels import *
 from .eval_defaults import *
 from .eval_random import *
+from .eval_tail import *
 from .plot_auto import *
+from .tran_shapley import *
 from .tran_summaries import *
 from .tran_tools import *
 
 # Integrate dfply tools
 # --------------------------------------------------
-from .dfply import Intention, dfdelegate, make_symbolic, var_in, convert_type
+from .dfply import Intention, dfdelegate, make_symbolic, convert_type
+from .dfply import var_in, is_nan, not_nan
 from .dfply import starts_with, ends_with, contains, matches, everything
 from .dfply import num_range, one_of, columns_between, columns_from, columns_to
 
@@ -65,7 +68,8 @@ from .dfply import summarize_each as tf_summarize_each
 
 # summary_functions.py
 from .dfply import mean, first, last, nth, n, n_distinct, IQR, quant
-from .dfply import colmin, colmax, median, var, sd
+from .dfply import colmin, colmax, colsum, median, var, sd, binomial_ci
+from .dfply import mse, rmse, ndme, rsq
 
 # transform.py
 from .dfply import mutate as tf_mutate
@@ -82,3 +86,4 @@ from .dfply import percent_rank, row_number
 
 # Add functionality to dfply
 from .string_helpers import *
+from .mutate_helpers import *
