@@ -25,7 +25,7 @@ from matplotlib.pyplot import hist
 ## Monte Carlo
 # --------------------------------------------------
 @curry
-def plot_scattermat(df, var=None):
+def plot_scattermat(df, var=None, hue=None):
     r"""Create a scatterplot matrix
 
     Create a scatterplot matrix. Often used to visualize a design (set of inputs
@@ -53,7 +53,7 @@ def plot_scattermat(df, var=None):
         raise ValueError("Must provide input columns list as keyword var")
 
     ## Plot
-    return pairplot(data=df, vars=var)
+    return pairplot(data=df, vars=var, hue=hue)
 
 
 @pipe
